import java.util.*;


/**
	* Create a vector with all the particules sorted by increasing next move time. It also manage all the events (move, addparticle, evaporation)
*/
public class EventManager {

	private Vector eventVector;	// contient toutes les particules tries par date croissante
	private static double time;	// temps actuel : le nextMoveTime de la dernire particule ayant boug
	private int evap=0;		//sert  savoir si une particule s'est vapor
	private  double tauArrival=1.;	//le paramtre de la loi de poisson dterminant les temps d'arrive des aprticules sur le grain.
	private double timeAux;		
	private double rand;		//un nombre alatoire
	private Random generator;	//un generateur de nombre alatoire
	private Random cellGenerator;	//un autre generateur de nombre alatoires 
	private Vector AllCells;	//vecteur contenant toutes les cellules
	private int indexAux;		
	private int numberOfCells;	// nombre de cellules sur le grain
	private Cell root;		//cellule univers : elle reprsente le milieu exterieur. Elle est le point de transit entre l'exterieur et le grain pour les particules
	private double arrivaltime;	// le temps d'arrive sur le grain de la dernire particules
	private Vector xyi, xy0;	
	private double n;		
	private double densite;		// la densit du milieu interstellaire
	private int particleNumber=0;	// le nombre de particules sur le grain
	private double ksurmH=0.0083;//Constante de Boltzman en J/K divis par masse d'un atome d'hydrogne en kg
	private double Vthermique ;	//vitesse thermique des atomes d'hydrogne du milieu exterieur.
	private double tau;


 /** 
  * default constructor
  */

   public EventManager () {

     eventVector = new Vector();
   }

 /**
  * particular constructor : a given number of particle is put in eventVector with a poissonian time of arrival.
  */


   public EventManager (int partNumber, Grid currentGrid, double density, double Tex, double Tgrain, double diameter, double densisite) {

	
	eventVector = new Vector();
	time = 0.;
	root=currentGrid.getRoot();
	densite=density;
	particleNumber=partNumber; 	
	Vthermique=Math.sqrt((8*ksurmH*Tex)/(Math.PI));
	
	tau=Math.sqrt(1/(densisite*2*ksurmH*Tgrain))*Math.exp(100/Tgrain); //la temperature du grain est ici celle de l'exterieur

	generator = new Random ();

	cellGenerator = new Random ();

	
	AllCells = currentGrid.getCells();  // vector de toutes les cellules sur le grain

	numberOfCells = AllCells.size();       // nombre total de cellules sur le grain
	arrivaltime=0;
	
	for (int i=0;i<partNumber;i++) {
		
	
		addParticle();

		sortEventVector();


	}

	if (density==0){  
		tauArrival=1;
		arrivaltime=1;

	}
	else{  
		tauArrival=Math.pow(10,6)/(density*Math.PI*diameter*diameter*Vthermique/4);
		arrivaltime= ((Particle)eventVector.elementAt(eventVector.size()-1)).getNextMoveTime();
	}
System.out.println("Vthermique= " + Vthermique+" tau= "+tau+"tauarrival= "+tauArrival);
 
}


 /** 
  * @return eventVector 
  */

	public Vector getEventVector () {
    
	return eventVector;
	}


	public double getTime(){

		return time;
	}

	public int getEvap () {
	
	return evap;
	}


 /** 
  * Add a new particle in the root cell ready to move on the grain
  */       

	public void addParticle () {      

		if (arrivaltime==0) {
			rand = generator.nextDouble();
			timeAux = time - Math.log(rand) * tauArrival;}
			
		else {
			timeAux=arrivaltime;
		}
		if(densite==0){
		particleNumber=particleNumber+1;
		}

		
		Particle newParticle = new Particle(timeAux,(Cell) root, tau);
		rand = generator.nextDouble();
		int d=0;
		double intervalle=0;
		int i=0;
		while (d==0) { 
			
			intervalle=intervalle+((Cell) AllCells.elementAt(i)).getArrival();
			i=i+1;

			if (intervalle>rand) {
				d=1;
			}
		}
		newParticle.setNextCell((Cell) AllCells.elementAt(i-1));
		newParticle.setNextCell((Cell) AllCells.elementAt(i-1));
		eventVector.insertElementAt(newParticle,0);
		sortEventVector();
	}


 /** 
  * Move the first particle and change eventVector
  */

	public void move () {

		Particle particleAux = (Particle) eventVector.elementAt(0);  // the current particle is the first in the list
		evap=0;
		time = particleAux.getNextMoveTime();
	
		if (particleAux.getNextCell().getOccupied() != -1) {
		
			particleAux.getCurrentCell().setOccupied(-1);			
			evaporation(particleAux.getNextCell());
			if (eventVector.size()==0 && densite==0){
				addParticle();
				addParticle();
			}
			
		}       

		else {
     
			particleAux.getNextCell().setOccupied(particleAux.getIndex()+1);// on déclare la nouvelle cellule comme occupée
			particleAux.getCurrentCell().setOccupied(-1);                 // on déclare l'ancienne cellule comme libre

			particleAux.move();    // this give a new date and a new cell for the current particle

			sortEventVector();
		}
		//une fois que la particule est arrive sur le grain, on en rajoute une sur la cellule univers mais seulement si la densit est non nulle
		if (time==arrivaltime && densite!=0) {
			arrivaltime=time-Math.log(rand)*tauArrival;
			particleNumber=particleNumber+1;
			rand = generator.nextDouble();
			
			addParticle();
		}
	}



 /** 
  * destroy in the eventVector the first particle and the one that is on the cell getOccupied
  */

	public void evaporation (Cell getOccupiedCell) {
 		
		particleNumber=particleNumber-2;

		for (int i=1; i< eventVector.size(); i++){
	

			Cell CellAux=((Particle) eventVector.elementAt(i)).getCurrentCell ();
	

			if (CellAux==getOccupiedCell)  {
				evap=1; 
				((Cell) ((Particle) eventVector.elementAt(0)).getNextCell ()).setOccupied(-1);
				eventVector.removeElementAt(i);    // destroy the particle on the getOccupied cell
				eventVector.removeElementAt(0);
			}

		}
	}

 /** 
  * eventVector is sorted by increasing date
  */



	public void sortEventVector() {

	


	
		int nombre_elements = eventVector.size();

		if (nombre_elements > 1) {
			for (int i = 0; i < nombre_elements - 1; i++) {
				Particle current_element = (Particle) eventVector.elementAt(i);
				Particle next_element = (Particle) eventVector.elementAt(i+1);
				double t0 = current_element.getNextMoveTime();
				double t1 = next_element.getNextMoveTime();
				if (t0 > t1) {
					// exchange the two elements
					this.eventVector.setElementAt(next_element, i);
					this.eventVector.setElementAt(current_element, i + 1);				
				}
				else
					break;
			}			
		}
		else {
		
		}		
	}

	public int getParticleNumber(){
		return particleNumber;

}
}