// Vect class
// Creation : 18/11/2008
// Last modification : 20/01/2009


import java.lang.Math.*;
import java.util.Vector;


/** 
 * Class representing a vector (geometric)
 */
public class Vect {

    private Point P1;
    private Point P2;



    /**
     * Constructor of a vector PQ defined by 2 points
     * @param P Point #1 
     * @param Q Point #2
     */
    public Vect (Point P, Point Q) {
	P1 = P;
	P2 = Q;
    }



    /** 
     * Returns the #1 point of the vector
     * @return P1
     */
    public Point getP1() {
	return P1;
    }



    /** 
     * Returns the #2 point of the vector
     * @return P2
     */
    public Point getP2() {
	return P2;
    }



    /** 
     * Returns the cartesian coordinates of the vector
     * @return Vector (X,Y,Z)
     */
    public Vector getCoordinates() {
	Point A = this.getP1();
	Point B = this.getP2();
	Vector pos = new Vector(3);

	Double x2 = (Double)((Point)B).getCartCoordinates().elementAt(0);
	Double x1 = (Double)((Point)A).getCartCoordinates().elementAt(0);
	double x = x2.doubleValue() - x1.doubleValue();

	Double y2 = (Double)((Point)B).getCartCoordinates().elementAt(1);
	Double y1 = (Double)((Point)A).getCartCoordinates().elementAt(1);
	double y = y2.doubleValue() - y1.doubleValue();


	Double z2 = (Double)((Point)B).getCartCoordinates().elementAt(2);
	Double z1 = (Double)((Point)A).getCartCoordinates().elementAt(2);
	double z = z2.doubleValue() - z1.doubleValue();

	pos.addElement(new Double(x));
	pos.addElement(new Double(y));
	pos.addElement(new Double(z));

	return pos;
    }
    


    /** 
     * Returns the norm of the vector
     * @return Norm of the vector
     */
    public double norm() {
	return Math.sqrt(this.scalar(this));
    }
    


    /** 
     * Returns the sum of 2 vectors
     * @param V Vector to add
     * @return Sum of the 2 vectors
     */
    public Vect sum(Vect V) {
	Vector A = this.getCoordinates();
	Vector B = V.getCoordinates();

	Double x2 = (Double)((Vector)B).elementAt(0);
	Double x1 = (Double)((Vector)A).elementAt(0);

	Double y2 = (Double)((Vector)B).elementAt(1);
	Double y1 = (Double)((Vector)A).elementAt(1);

	Double z2 = (Double)((Vector)B).elementAt(2);
	Double z1 = (Double)((Vector)A).elementAt(2);

	double x = x1.doubleValue() + x2.doubleValue();
	double y = y1.doubleValue() + y2.doubleValue();
	double z = z1.doubleValue() + z2.doubleValue();

	Point P = new Point(x,y,z,0);
	Point O = new Point(0,0,0,0);

	Vect result = new Vect(O,P);
	return result;	
    }
    


    /** 
     * Returns the difference of 2 vectors
     * @param V2 Vector from which the vector is substracted
     * @return Substracted vector
     */
    public Vect diff(Vect V2) {
	Vector A = this.getCoordinates();
	Vector B = V2.getCoordinates();

	Double x2 = (Double)((Vector)B).elementAt(0);
	Double x1 = (Double)((Vector)A).elementAt(0);

	Double y2 = (Double)((Vector)B).elementAt(1);
	Double y1 = (Double)((Vector)A).elementAt(1);

	Double z2 = (Double)((Vector)B).elementAt(2);
	Double z1 = (Double)((Vector)A).elementAt(2);

	double x = x2.doubleValue() - x1.doubleValue();
	double y = y2.doubleValue() - y1.doubleValue();
	double z = z2.doubleValue() - z1.doubleValue();

	Point P = new Point(x,y,z,0);
	Point O = new Point(0,0,0,0);

	Vect result = new Vect(O,P);
	return result;	
    }
    


    /** 
     * Returns a proportionnal vector normalized by a constant 
     * @param d multiplying constant
     * @return Proportionnal vector
     */
    public Vect propVect(double d) {

	    Vector A = this.getCoordinates();

	    Double x1 = (Double)((Vector)A).elementAt(0);
	    Double y1 = (Double)((Vector)A).elementAt(1);
	    Double z1 = (Double)((Vector)A).elementAt(2);
	    
	    double x = x1.doubleValue() * d;
	    double y = y1.doubleValue() * d;
	    double z = z1.doubleValue() * d;
	    
	    Point P = new Point(x,y,z,0);
	    Point O = new Point(0,0,0,0);
    
	    Vect result = new Vect(O,P);
	    return result;	
    }
    


    /** 
     * Returns the vector normalized
     * @return Normalized vector
     */
    public Vect normalize() {
    
	double n = this.norm();
	if (n < 1E-10) {
	    System.out.println("Error in Vect.java : Division by 0");
	    System.exit(1);
	}
	Vect result = this.propVect(1./n);
	return result;	
    }



    /** 
     * Returns the scalar product with another vecetor
     * @return Scalar product
     */
    public double scalar(Vect V) {
	Vector A = this.getCoordinates();
	Vector B = V.getCoordinates();

	Double x1 = (Double)((Vector)B).elementAt(0);
	Double x2 = (Double)((Vector)A).elementAt(0);
	double x = x2.doubleValue() * x1.doubleValue();

	Double y1 = (Double)((Vector)B).elementAt(1);
	Double y2 = (Double)((Vector)A).elementAt(1);
	double y = y2.doubleValue() * y1.doubleValue();


	Double z1 = (Double)((Vector)B).elementAt(2);
	Double z2 = (Double)((Vector)A).elementAt(2);
	double z = z2.doubleValue() * z1.doubleValue();

	double result =  x + y + z;
	return result;
    }
    


    /** 
     * Returns the vectorial product with another vector
     * @return Vectorial product
     */
    public Vect cross(Vect V) {
	Vector A = this.getCoordinates();
	Vector B = V.getCoordinates();

	Double x2 = (Double)((Vector)B).elementAt(0);
	Double x1 = (Double)((Vector)A).elementAt(0);

	Double y2 = (Double)((Vector)B).elementAt(1);
	Double y1 = (Double)((Vector)A).elementAt(1);

	Double z2 = (Double)((Vector)B).elementAt(2);
	Double z1 = (Double)((Vector)A).elementAt(2);

	double x = y1.doubleValue() * z2.doubleValue() - y2.doubleValue() * z1.doubleValue();
	double y = z1.doubleValue() * x2.doubleValue() - z2.doubleValue() * x1.doubleValue();
	double z = x1.doubleValue() * y2.doubleValue() - x2.doubleValue() * y1.doubleValue();

	Point P = new Point(x,y,z,0);
	Point O = new Point(0,0,0,0);

	Vect result = new Vect(O,P);
	return result;
    }
    


    /** 
     * Displays the cartesian coordinates of the vector
     */
    public void printCoordinates() {
	System.out.println("x= " + this.getCoordinates().elementAt(0) +  " y= " + this.getCoordinates().elementAt(1) + " z= " + this.getCoordinates().elementAt(2) + "\n");
    }


}



